#!/usr/bin/python3
# This file is dedicated to the public domain
# Config file shared between doc_client and doc_server

# pylint: disable=unused-variable, missing-docstring
# Default server host and port and dictionary
DEFAULT_HOST = "localhost"
DEFAULT_PORT = 60604
DEFAULT_DICT = 'dicts/de-mixed.dic'
DE_MISC_DICT = 'dicts/de-misc.dic'
